class TerraformBlock:
    def __init__(self):
        """
        Initializes a new instance of the TerraformBlock class.

        Attributes:
            file_path (str): The path of the file where the block is defined
        """
        self.file_path: str = ""



from typing import Self
from pyhcl_fancy.blocks.terraform_block import TerraformBlock


class Node:
    def __init__(self):
        """
        Initializes a new instance of the Node class.

        Attributes:
            blocks (list): The content of a given file, defined as a list of Terraform blocks.
            is_directory (bool): True if the node is a directory, otherwise False.
            relative_file_path (str): The relative path of the file or directory represented by the node.
            parent (Node): The parent node in the collection tree.
            children (list[Node]): A list of child nodes in the collection tree.
            is_root (bool): True if the node is the root node of the collection tree, otherwise False.
            is_leaf (bool): True if the node is a leaf node in the collection tree, otherwise False.
        """
        self.blocks: list[TerraformBlock] = []
        self.is_directory: bool = False
        self.relative_file_path: str = ""
        self.parent: Node = None
        self.children: list[Node] = []
        self.is_root: bool = False
        self.is_leaf: bool = True
        self.submodule_state_path: str = None


    def add_child(self, child: Self) -> None:
        """
        Adds a child node to the current node's list of children.

        Args:
            child (Node): The child node to be added to this node's children list.

        Returns:
            None
        """
        self.children.append(child)
        child.parent = self
        self.is_leaf = False



from pyhcl_fancy.blocks.real.real_block import RealBlock
from pyhcl_fancy.collection_tree.node import Node


class DataBlock(RealBlock):
    def __init__(self):
        """
        Initializes a new instance of the DataBlock class.

        Attributes:
            data_type (str): The type of the data block.
            data_name (str): The name of the data block.

        Inherited Attributes:
            state_path (str): The state path of the block.
            count (int): The count of the block.
            for_each (list | dict): The for_each of the block.
            content (dict): The content of the block.
        """
        super().__init__()
        self.data_type: str = ""
        self.data_name: str = ""


    def parse(
        self, raw_data_dict: dict, data_file_path: str, parent_file_node: Node
    ) -> None:
        """
        Parses the raw_data_dict dictionary and sets the DataBlock's fields.

        Args:
            raw_data_dict (dict): The raw dictionary from the terraform file for the data block.
            data_file_path (str): The path of the file where the data block is defined.
            parent_file_node (Node): The parent node in the collection tree.

        Returns:
            None
        """

        self.data_type = list(raw_data_dict.keys())[0]
        self.data_name = list(raw_data_dict[self.data_type].keys())[0]
        self.file_path = data_file_path
        if parent_file_node.submodule_state_path is None:
            self.state_path = f"data.{self.data_type}.{self.data_name}"
        else:
            self.state_path = f"{parent_file_node.submodule_state_path}.data.{self.data_type}.{self.data_name}"
        for attribute in raw_data_dict[self.data_type][self.data_name]:
            self.content[attribute] = raw_data_dict[self.data_type][self.data_name][
                attribute
            ]



from pyhcl_fancy.blocks.terraform_block import TerraformBlock


class TerraformMetaBlock(TerraformBlock):
    def __init__(self):
        """
        Initializes a new instance of the TerraformMetaBlock class.

        Attributes:
            backend (dict): The backend configuration for Terraform.
            required_providers (list): A list of required providers for the Terraform configuration.
        """
        super().__init__()
        self.backend_type: str = ""
        self.backend_config: dict = {}
        self.required_providers: list[dict] = []
        self.options: dict = {}

    def convert_to_hcl(self) -> str:
        pass

    def parse(self, raw_meta_dict: dict, meta_file_path: str) -> str:
        """
        Parses the raw meta dictionary and updates the TerraformMetaBlock attributes.

        This function iterates over the settings in the raw meta dictionary and
        updates the corresponding attributes of the TerraformMetaBlock based on
        the setting type. It handles backend configurations by setting the backend
        type and configuration, and also processes required providers. Any other
        settings are stored in the options dictionary.

        Args:
            raw_meta_dict (dict): The dictionary containing raw meta settings.
            meta_file_path (str): The path to the file containing meta configuration.

        Returns:
            None
        """

        self.file_path = meta_file_path
        for setting in raw_meta_dict:
            match setting:
                case "backend":
                    if len(raw_meta_dict[setting]) > 1:
                        raise ValueError(
                            f"Invalid backend configuration: multiple backends found in {meta_file_path}"
                        )
                    self.backend_type = list(raw_meta_dict[setting][0].keys())[0]
                    self.backend_config = raw_meta_dict[setting][0][self.backend_type]
                case "required_providers":
                    self.required_providers = raw_meta_dict[setting]
                case _:
                    self.options[setting] = raw_meta_dict[setting]



from pyhcl_fancy.blocks.terraform_block import TerraformBlock
from typing import Any


class OutputBlock(TerraformBlock):
    def __init__(self):
        """
        Initializes a new instance of the OutputBlock class.

        Attributes:
            value (Any): The value of the output variable.
            description (str): The description of the output variable.
        """
        super().__init__()
        self.name: str = ""
        self.value: Any = None
        self.description: str = ""
        self.options: dict = {}


    def parse(self, raw_output_dict: dict, output_file_path: str) -> None:
        """
        Parses the raw_output_dict dictionary and sets the OutputBlock's fields.

        Args:
            raw_output_dict (dict): The raw dictionary from the terraform file for the output block.
            output_file_path (str): The path of the file where the output block is defined.

        Returns:
            None
        """

        self.name = list(raw_output_dict.keys())[0]
        self.file_path = output_file_path
        for attribute in raw_output_dict[self.name]:
            match attribute:
                case "value":
                    self.value = raw_output_dict[self.name][attribute]
                case "description":
                    self.description = raw_output_dict[self.name][attribute]
                case _:
                    self.options[attribute] = raw_output_dict[self.name][attribute]



from pyhcl_fancy.collection_tree.node import Node
from pyhcl_fancy.blocks.real.module.module_block import ModuleBlock
from pyhcl_fancy.collection_tree.exceptions import (
    FileNodeNotFoundError,
    DirectoryNodeNotFoundError,
    InvalidMoveLocationError,
)


class CollectionTree:
    def __init__(self):
        """
        Initializes a new instance of the CollectionTree class.

        Attributes:
            root (Node): The root node of the collection tree.
        """
        self.root: Node = None

    def add_root(self, node: Node) -> Node:
        """
        Adds a root node to the collection tree if one does not already exist.

        Args:
            node: The root node to add to the collection tree.

        Returns:
            The root node of the collection tree.
        """
        if self.root is None:
            self.root = node
        return self.root


    def find_directory_node(self, target_directory: str) -> Node:
        """
        Wrapper function to find a directory node in the collection tree.

        Args:
            target_directory (str): The path of the directory to search for.

        Returns:
            Node: The directory node if found, otherwise raises a DirectoryNodeNotFoundError.
        """
        found_node = self._find_directory_node(self.root, target_directory)
        if found_node is None:
            raise DirectoryNodeNotFoundError(
                f"Directory node for {target_directory} not found in the collection tree."
            )
        
        return found_node


    def _find_directory_node(self, node: Node, target_directory: str) -> Node:
        """
        Recursively searches for a directory node within the collection tree
        given a directory path.

        Args:
            node (Node): The current node to search in the collection tree.
            target_directory (str): The path of the directory to search for.

        Returns:
            Node: The directory node if found, otherwise None.
        """
        if node.is_directory and node.relative_file_path == target_directory:
            return node
        
        for child in node.children:
            found_node = self._find_directory_node(child, target_directory)
            if found_node is not None:
                return found_node
        
        return None
                

    def find_file_node(self, target_file: str) -> Node:
        """
        Wrapper function to find a file node in the collection tree.

        Args:
            target_file (str): The path of the file to search for.

        Returns:
            Node: The file node if found, otherwise raises a FileNodeNotFoundError.
        """
        found_node = self._find_file_node(self.root, target_file)
        if found_node is None:
            raise FileNodeNotFoundError(
                f"File node for {target_file} not found in the collection tree."
            )
        
        return found_node


    def _find_file_node(self, node: Node, target_file: str) -> Node:
        """
        Recursively searches for a file node within the collection tree
        given a file path.

        Args:
            node (Node): The current node to search in the collection tree.
            target_file (str): The path of the file to search for.

        Returns:
            Node: The file node if found, otherwise None.
        """
        if not node.is_directory and node.relative_file_path == target_file:
            return node

        for child in node.children:
            found_node = self._find_file_node(child, target_file)
            if found_node is not None:
                return found_node

        return None

    def move_node(self, source: Node, destination: Node, caller: ModuleBlock) -> Node:
        """
        Moves a node from one location to another in the collection tree.

        Args:
            source (Node): The node to be moved.
            destination (Node): The node to which the source node is to be moved.
            caller (ModuleBlock): The caller of the move operation.

        Returns:
            Node: A reference to the moved node.

        Raises:
            InvalidMoveLocationError: If the source and destination nodes have the same directory status or if the destination node is already the parent of the source node.
        """
        if source == destination:
            raise InvalidMoveLocationError("Cannot move a node to itself.")
        elif source.parent == destination:
            raise InvalidMoveLocationError(
                "The destination node is already the parent of the source node."
            )
        elif not source.is_directory ^ destination.is_directory:
            raise InvalidMoveLocationError(
                f"The source and destination nodes must not have the same directory status. Both nodes have directory status {source.is_directory}."
            )
        
        curr_parent = source.parent
        for child in curr_parent.children:
            if child == source:
                node_to_move = child

        # remove node from current parent and add to new parent
        curr_parent.children.remove(node_to_move)
        destination.add_child(node_to_move)
        
        # update the submodule state path 
        if destination.submodule_state_path == "":
            source.submodule_state_path = f"module.{caller.module_name}"
        else:
            source.submodule_state_path = f"{destination.submodule_state_path}.module.{caller.module_name}"

        return source



from pyhcl_fancy.blocks.terraform_block import TerraformBlock
from typing import Any


class VariableBlock(TerraformBlock):
    def __init__(self):
        """
        Initializes a new instance of the VariableBlock class.

        Attributes:
            variable_name (str): The name of the variable.
            description (str): The description of the variable.
            type (str): The type of the variable.
            default (Any): The default value of the variable.
            validation (dict): The validation rules for the variable.

        Inherited Attributes:
            file_path (str): The path of the file where the variable is defined.
        """
        super().__init__()
        self.variable_name: str = ""
        self.description: str = ""
        self.type: str = ""
        self.default: Any = None
        self.validation: dict = {}

    def convert_to_hcl(self) -> str:
        pass

    def parse(self, raw_variable_dict: dict, variable_file_path: str) -> None:
        """
        Parses the raw_variable_dict dictionary and sets the VariableBlock's fields.

        Args:
            raw_variable_dict (dict): The raw dictionary from the terraform file for the variable block.
            variable_file_path (str): The path of the file where the variable block is defined.

        Returns:
            None
        """

        self.variable_name = list(raw_variable_dict.keys())[0]
        self.file_path = variable_file_path
        for attribute in raw_variable_dict[self.variable_name]:
            match attribute:
                case "description":
                    self.description = raw_variable_dict[self.variable_name][attribute]
                case "type":
                    self.type = raw_variable_dict[self.variable_name][attribute]
                case "default":
                    self.default = raw_variable_dict[self.variable_name][attribute]
                case "validation":
                    self.validation = raw_variable_dict[self.variable_name][attribute]



from pyhcl_fancy.blocks.real.real_block import RealBlock
from pyhcl_fancy.collection_tree.node import Node


class ModuleBlock(RealBlock):
    def __init__(self):
        """
        Initializes a new instance of the ModuleBlock class.

        Attributes:
            module_name (str): The name of the module.
            module_source (str): The source of the module.
        """
        super().__init__()
        self.module_name: str = ""
        self.module_source: str = ""
        self.module_version: str = ""


    def parse(
        self, raw_module_dict: dict, module_file_path: str, parent_file_node: Node
    ) -> None:
        """
        Parses the raw_module_dict to set the ModuleBlock's fields.

        Args:
            raw_module_dict (dict): The raw dictionary from the terraform file for the module block.
            module_file_path (str): The path of the file where the module block is defined.
            parent_file_node (Node): The parent node in the collection tree.

        Returns:
            None
        """

        self.module_name = list(raw_module_dict.keys())[0]
        self.file_path = module_file_path
        if parent_file_node.submodule_state_path is None:
            self.state_path = f"module.{self.module_name}"
        else:
            self.state_path = f"{parent_file_node.submodule_state_path}.module.{self.module_name}"
        for attribute in raw_module_dict[self.module_name]:
            match attribute:
                case "source":
                    self.module_source = raw_module_dict[self.module_name][attribute]
                case "version":
                    self.module_version = raw_module_dict[self.module_name][attribute]
                case _:
                    self.content[attribute] = raw_module_dict[self.module_name][
                        attribute
                    ]



from pyhcl_fancy.blocks.terraform_block import TerraformBlock


class LocalBlock(TerraformBlock):
    def __init__(self):
        """
        Initializes a new instance of the LocalBlock class.

        Attributes:
            content (dict): The content of the locals block.
        """
        super().__init__()
        self.content: dict = {}

    def convert_to_hcl(self) -> str:
        pass

    def parse(self, raw_locals_dict: dict, locals_file_path: str) -> str:
        """
        Parses the raw locals dictionary and sets the content of the LocalBlock.

        Args:
            raw_locals_dict (dict): The raw locals dictionary to parse.
            locals_file_path (str): The path of the file containing the locals block.

        Returns:
            str: The parsed content of the locals block.
        """
        self.file_path = locals_file_path
        for local_dict in raw_locals_dict:
            self.content.update(local_dict)



import pathlib
import hcl2


def _open_all_tf_files(directory_path: str) -> dict:
    """
    Recursively opens all Terraform (.tf) files in the given directory and
    loads their content into a dictionary.

    Args:
        directory_path (str): The path to the directory containing Terraform files.

    Returns:
        dict: A dictionary mapping each Terraform file path to its parsed content.
    """
    tf_dir = pathlib.Path(directory_path)
    tf_file_content = {}

    for tf_file in tf_dir.rglob("./*.tf"):
        with open(tf_file, "r") as f:
            tf_file_content[str(tf_file)] = hcl2.load(f)

    return tf_file_content



from pyhcl_fancy.blocks.terraform_block import TerraformBlock


class RealBlock(TerraformBlock):
    def __init__(self):
        """
        Initializes a new instance of the RealBlock class.

        Attributes:
            state_path (str): The state path of the block.
            count (int): The count of the block.
            for_each (list | dict): The for_each of the block.
            content (dict): The content of the block.
        """
        super().__init__()
        self.state_path: str = ""
        self.count: int = None
        self.for_each: list | dict = None
        self.content: dict = {}



from pyhcl_fancy.blocks.terraform_block import TerraformBlock


class ProviderBlock(TerraformBlock):
    def __init__(self):
        """
        Initializes a new instance of the ProviderBlock class.

        Attributes:
            type (str): The type of the provider.
            region (str): The region for the provider.
            alias (str): The alias of the provider.
            assume_role (str): The assume role for the provider.
            default_tags (dict): The default tags for the provider.
            options (dict): The options for the provider.
        """
        super().__init__()
        self.type: str = ""
        self.region: str = ""
        self.alias: str = ""
        self.assume_role: dict = {}
        self.default_tags: dict = {}
        self.options: dict = {}


    def parse(self, raw_provider_dict: dict, provider_file_path: str) -> None:
        """
        Parses the raw_provider_dict to set the ProviderBlock's fields.

        Args:
            raw_provider_dict (dict): The raw dictionary from the terraform file for the provider block.
            provider_file_path (str): The path of the file where the provider block is defined.

        Returns:
            None
        """

        self.type = list(raw_provider_dict.keys())[0]
        self.file_path = provider_file_path
        for attribute in raw_provider_dict[self.type]:
            match attribute:
                case "region":
                    self.region = raw_provider_dict[self.type][attribute]
                case "alias":
                    self.alias = raw_provider_dict[self.type][attribute]
                case "assume_role":
                    if len(raw_provider_dict[self.type][attribute]) > 1:
                        raise ValueError(
                            f"Invalid assume role configuration: multiple assume roles found in {provider_file_path}"
                        )
                    self.assume_role = raw_provider_dict[self.type][attribute][0]
                case "default_tags":
                    self.default_tags = raw_provider_dict[self.type][attribute][0]["tags"]
                case _:
                    self.options[attribute] = raw_provider_dict[self.type][attribute]



from pathlib import Path

from pyhcl_fancy.collection_tree.node import Node
from pyhcl_fancy.collection_tree.tree import CollectionTree
from pyhcl_fancy.collection_tree.exceptions import DirectoryNodeNotFoundError


from pyhcl_fancy.blocks.real.data.data_block import DataBlock
from pyhcl_fancy.blocks.real.module.module_block import ModuleBlock
from pyhcl_fancy.blocks.real.resource.resource_block import ResourceBlock
from pyhcl_fancy.blocks.reference.meta.meta_block import TerraformMetaBlock
from pyhcl_fancy.blocks.reference.output.output_block import OutputBlock
from pyhcl_fancy.blocks.reference.provider.provider_block import ProviderBlock
from pyhcl_fancy.blocks.reference.local.local_block import LocalBlock
from pyhcl_fancy.blocks.reference.variable.variable_block import VariableBlock

from pyhcl_fancy.parser.utils import _open_all_tf_files
from pyhcl_fancy.parser.exceptions import UnexpectedTerraformBlockError


class FancyParser:
    def __init__(self, terraform_directory: str):
        """
        Initializes a new instance of the FancyParser class.

        Args:
            terraform_directory (str): The path to the directory containing Terraform files.

        Attributes:
            terraform_directory (str): The path to the directory containing Terraform files.
            terraform_content (dict): A dictionary mapping file paths to their parsed content.
            collection_tree (CollectionTree): The collection tree containing the directory and file nodes.
        """

        self.terraform_directory = terraform_directory
        self.terraform_content: dict = {}
        self.collection_tree: CollectionTree = CollectionTree()

    def construct_empty_tree(self) -> None:
        """
        Constructs an empty collection tree from the Terraform files.

        Iterates through the Terraform content, setting up the directory
        and file nodes in the collection tree. It initializes the root node
        and adds directory and file nodes based on the file paths in the
        Terraform content. If a directory node is not found, it creates a new
        one and adds it to the collection tree.
        """

        root = self._set_tree_root()

        for file in self.terraform_content:
            # if file path minus file name is the given directory, file is root level
            if "/".join(file.split("/")[:-1]) == self.terraform_directory:
                directory_node = root
            else:
                try:
                    # directory path is everything but the last element
                    directory_path = file.split("/")[:-1]
                    directory_node = self.collection_tree.find_directory_node(
                        directory_path
                    )
                # if directory node isn't found, create it
                except DirectoryNodeNotFoundError:
                    directory_node = Node()
                    directory_node.is_directory = True
                    directory_node.relative_file_path = "/".join(directory_path)
                    directory_node.parent = root
                    root.add_child(directory_node)

            # add the file node to the directory node
            file_node = Node()
            file_node.relative_file_path = file
            directory_node.add_child(file_node)

    def parse(self) -> None:
        """
        Parses the Terraform content and constructs the collection tree.

        This function iterates over all files in the Terraform content and
        parses the blocks in each file. It uses a switch case to apply a ruleset
        based on the type of block. The ruleset is as follows:

        * For each module block, it initializes a new ModuleBlock and parses
          the raw module dictionary. It then checks if the module source points
          to a submodule, and if so, moves the submodule's directory node to the
          calling module's file node.
        * For each resource block, it initializes a new ResourceBlock and parses
          the raw resource dictionary.
        * For each data block, it initializes a new DataBlock and parses the raw
          data dictionary.
        * For each output block, it initializes a new OutputBlock and parses the
          raw output dictionary.
        * For each variable block, it initializes a new VariableBlock and parses
          the raw variable dictionary.
        * For each local block, it initializes a new LocalBlock and parses the
          raw locals dictionary.
        * For each provider block, it initializes a new ProviderBlock and parses
          the raw provider dictionary.
        * For each terraform block, it initializes a new TerraformMetaBlock and
          parses the raw meta dictionary.
        * If an unexpected block type is found, it raises an UnexpectedTerraformBlockError.

        Returns:
            None

        Notes:
            This function represents the primary API for this library and will
            be the main entrypoint that calling applications will use.
        """
        self._read_tf_files()
        self.construct_empty_tree()

        for file in self.terraform_content:
            file_node = self.collection_tree.find_file_node(
                file
            )

            for block_type in self.terraform_content[file]:
                # switch case to apply a ruleset based on the type of block
                match block_type:
                    case "module":
                        for module in self.terraform_content[file][block_type]:
                            module_block = ModuleBlock()
                            module_block.parse(
                                raw_module_dict=module,
                                module_file_path=file,
                                parent_file_node=file_node,
                            )
                            file_node.blocks.append(module_block)

                        # module source points to a submodule, move that submodules directory node to the calling module's file node
                        if Path(module_block.module_source).is_dir():
                            submodule_directory_node = (
                                self.collection_tree.find_directory_node(
                                    module_block.module_source,
                                )
                            )
                            self.collection_tree.move_node(
                                submodule_directory_node, file_node, module_block
                            )

                    case "resource":
                        for resource in self.terraform_content[file][block_type]:
                            resource_block = ResourceBlock()
                            resource_block.parse(
                                raw_resource_dict=resource,
                                resource_file_path=file,
                                parent_file_node=file_node,
                            )
                            file_node.blocks.append(resource_block)
                    case "data":
                        for data in self.terraform_content[file][block_type]:
                            data_block = DataBlock()
                            data_block.parse(
                                raw_data_dict=data,
                                data_file_path=file,
                                parent_file_node=file_node,
                            )
                            file_node.blocks.append(data_block)
                    case "output":
                        for output in self.terraform_content[file][block_type]:
                            output_block = OutputBlock()
                            output_block.parse(
                                raw_output_dict=output, output_file_path=file
                            )
                            file_node.blocks.append(output_block)
                    case "variable":
                        for variable in self.terraform_content[file][block_type]:
                            variable_block = VariableBlock()
                            variable_block.parse(
                                raw_variable_dict=variable, variable_file_path=file
                            )
                            file_node.blocks.append(variable_block)
                    case "locals":
                        local_block = LocalBlock()
                        local_block.parse(
                            raw_locals_dict=self.terraform_content[file][block_type],
                            locals_file_path=file,
                        )
                        file_node.blocks.append(local_block)
                    case "provider":
                        for provider in self.terraform_content[file][block_type]:
                            provider_block = ProviderBlock()
                            provider_block.parse(
                                raw_provider_dict=provider, provider_file_path=file
                            )
                            file_node.blocks.append(provider_block)
                    case "terraform":
                        for meta in self.terraform_content[file][block_type]:
                            meta_block = TerraformMetaBlock()
                            meta_block.parse(raw_meta_dict=meta, meta_file_path=file)
                            file_node.blocks.append(meta_block)
                    case _:
                        raise UnexpectedTerraformBlockError(
                            f"Found unexpected Terraform block type {block_type} in file {file}"
                        )

    def _read_tf_files(self) -> None:
        """
        Reads and loads all Terraform files from the specified directory.

        This function utilizes the `_open_all_tf_files` utility to open
        and parse all Terraform files in the provided directory. The parsed
        content is then stored in the `terraform_content` attribute as a
        dictionary mapping file paths to their content.

        Returns:
            None
        """
        self.terraform_content = _open_all_tf_files(self.terraform_directory)

    def _set_tree_root(self) -> Node:
        """
        Initializes the root node of the collection tree.

        This function creates a Node object that is marked as the root of the
        collection tree. The root node is a directory node with its file path
        set to the provided Terraform directory.

        Returns:
            Node: The root node of the collection tree.
        """
        root = Node()
        root.is_root = True
        root.is_directory = True
        root.submodule_state_path = ""
        root.relative_file_path = self.terraform_directory
        return self.collection_tree.add_root(root)



from pyhcl_fancy.blocks.real.real_block import RealBlock
from pyhcl_fancy.collection_tree.node import Node


class ResourceBlock(RealBlock):
    def __init__(self):
        """
        Initializes a new instance of the ResourceBlock class.

        Attributes:
            resource_type (str): The type of the resource.
            resource_name (str): The name of the resource.

        Inherited Attributes:
            state_path (str): The state path of the block.
            count (int): The count of the block.
            for_each (list | dict): The for_each of the block.
            content (dict): The content of the block.
        """
        super().__init__()
        self.resource_type: str = ""
        self.resource_name: str = ""


    def parse(
        self, raw_resource_dict: dict, resource_file_path: str, parent_file_node: Node
    ) -> None:
        """
        Parses the raw_resource_dict to set the ResourceBlock's fields.

        Args:
            raw_resource_dict (dict): The raw dictionary from the terraform file for the resource block.
            resource_file_path (str): The path of the file where the resource block is defined.
            file_node (Node): The node representing the file in the collection tree.

        Returns:
            None
        """

        self.resource_type = list(raw_resource_dict.keys())[0]
        self.resource_name = list(raw_resource_dict[self.resource_type].keys())[0]
        if parent_file_node.submodule_state_path is None:
            self.state_path = f"{self.resource_type}.{self.resource_name}"
        else:
            self.state_path = f"{parent_file_node.submodule_state_path}.{self.resource_type}.{self.resource_name}"
        self.file_path = resource_file_path
        for attribute in raw_resource_dict[self.resource_type][self.resource_name]:
            self.content[attribute] = raw_resource_dict[self.resource_type][
                self.resource_name
            ][attribute]
